<?php
namespace Odhen\API\Remote\TEF;

use Zeedhi\Framework\Remote\cURLRequest;
use Odhen\API\Remote\TEF\Transaction;
use Odhen\API\Util\Exception;

class TEF
{
  protected $curl;
  protected $util;
  protected $entityManager;
  const CREDIT = '1';
  const DEBIT = '2';
  public function __construct(cURLRequest $curl, \Odhen\API\Util\Util $util, \Doctrine\ORM\EntityManager $entityManager, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager)
  {
    $this->curl = $curl;
    $this->entityManager = $entityManager;
    $this->util = $util;
    $this->curl->setHeaders(array(
      cURLRequest::CONTENT_TYPE_HEADER => cURLRequest::CONTENT_TYPE_APPLICATION_JSON
    ));
    $this->systemPath = $instanceManager->getParameter('SYSTEM_PATH');
  }
  const EVENT_CONFIGURA_TEF = '/configure';
  const EVENT_INICIA_TRANSACAO = '/start-transaction';
  const EVENT_CONTINUA_TRANSACAO = '/continue-transaction';
  const EVENT_FINALIZA_TRANSACAO = '/finish-transaction';
  const SALVA_LOGS = true;

  public function configuraTransacaoTef($CDFILIAL, $CDCAIXA, $NRORG = '1')
  {
    try {
      $params = array(
        'CDFILIAL' => $CDFILIAL,
        'CDCAIXA' => $CDCAIXA,
        'NRORG' => $NRORG
      );
      $dadosTef = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_TEF", $params);
      $transaction = new Transaction();
      $transaction->setEndSitef($dadosTef['DSENDIPSITEF']);
      $transaction->setStoreId($dadosTef['CDLOJATEF']);
      $transaction->setTerminalId($dadosTef['CDTERTEF']);
      $paramsTEF = array(
        'endSitef' => $transaction->getEndSitef(),
        'storeId' => $transaction->getStoreId(),
        'terminalId' => $transaction->getTerminalId()
      );
      $this->setClientIpCurl();
      $respostaTEF = $this->curl->request(self::EVENT_CONFIGURA_TEF, $paramsTEF);
      $respostaTEF = json_decode($respostaTEF, true);
      if (!$respostaTEF['error']) {
        return $this->trataRetornoConfiguracao($respostaTEF['result']);
      } else {
        return $respostaTEF;
      }
    } catch (\Exception $e) {
      Exception::logException($e, Exception::LOG_TYPES['EXCEPTION_LOG']);
      return array(
        'error' => true,
        'message' => 'Mensagem: ' . $e->getMessage()
      );
    }
  }

  public function iniciaCancelamentoTef($typeCancellation, $totalSale, $CDOPERADOR)
  {
    try {
      $funcao = $typeCancellation === self::CREDIT ? 210 : 211;
      $data = date('Ymd');
      $horario = date('His');
      $paramsInicia = array(
        'startFunction' => $funcao,
        'value' => $totalSale,
        'text' => '',
        'date' => $data,
        'time' => $horario,
        'operator' => $CDOPERADOR,
        'additionalParams' => null
      );
      $this->setClientIpCurl();
      $retornoInicia = $this->curl->request(self::EVENT_INICIA_TRANSACAO, $paramsInicia);
      $retornoInicia = json_decode($retornoInicia, true);
      $retornoInicia = json_decode($retornoInicia['result'], true);
      if ($retornoInicia['retornoDll'] != 10000) {
        return array(
          'error' => true,
          'message' => 'Falha ao iniciar transação. Código de retorno: ' . $retornoInicia['retornoDll']
        );
      } else {
        return array(
          'error' => false,
          'retornoInicia' => $retornoInicia
        );
      }
    } catch (\Exception $e) {
      Exception::logException($e, Exception::LOG_TYPES['EXCEPTION_LOG']);
      return array(
        'error' => true,
        'message' => 'Mensagem: ' . $e->getMessage()
      );
    }
  }

  public function iniciaTransacaoTef($IDTIPORECE, $totalSale, $CDOPERADOR)
  {
    try {
      $funcao = $this->trataRecebimentoDll($IDTIPORECE);
      $valor = $totalSale;
      $cupomFiscal = '';
      $data = date('Ymd');
      $horario = date('His');
      $operador = $CDOPERADOR;
      if ($funcao == 2) {
        //Restrições do TAA para TEF - Cartão DEBITO
        $paramsAdicionais = '[10;17;18;19;24;27;28;29;31;32;33;40]'; //16;
      } elseif ($funcao == 3) {
        //Restrições do TAA para TEF - Cartão CREDITO
        $paramsAdicionais = '[10;24;27;28;29;31;32;33;36;40;44]';
      }
      $paramsInicia = array(
        'startFunction' => $funcao,
        'value' => $valor,
        'text' => $cupomFiscal,
        'date' => $data,
        'time' => $horario,
        'operator' => $operador,
        'additionalParams' => $paramsAdicionais
      );
      $this->setClientIpCurl();
      $retornoInicia = $this->curl->request(self::EVENT_INICIA_TRANSACAO, $paramsInicia);
      $retornoInicia = json_decode($retornoInicia, true);
      $retornoInicia = json_decode($retornoInicia['result'], true);
      if ($retornoInicia['retornoDll'] != 10000) {
        return array(
          'error' => true,
          'message' => 'Falha ao iniciar transação. Código de retorno: ' . $retornoInicia['retornoDll']
        );
      } else {
        return array(
          'error' => false,
          'retornoInicia' => $retornoInicia
        );
      }
    } catch (\Exception $e) {
      Exception::logException($e, Exception::LOG_TYPES['EXCEPTION_LOG']);
      return array(
        'error' => true,
        'message' => 'Mensagem: ' . $e->getMessage()
      );
    }
  }

  private function trataRecebimentoDll($IDTIPORECE)
  {
    if ($IDTIPORECE == '1') {
      return 3;
    } else {
      return 2;
    }
  }

  private function trataRetornoConfiguracao($retornoConfigura)
  {
    switch ($retornoConfigura) {
      case 0:
        $retorno = array(
          'error' => false,
          'message' => 'Não ocorreu erro'
        );
        break;
      case 1:
        $retorno = array(
          'error' => true,
          'message' => 'Endereço IP inválido ou não resolvido.'
        );
        break;
      case 2:
        $retorno = array(
          'error' => true,
          'message' => 'Código da loja inválido.'
        );
        break;
      case 3:
        $retorno = array(
          'error' => true,
          'message' => 'Código de terminal invalido.'
        );
        break;
      case 6:
        $retorno = array(
          'error' => true,
          'message' => 'Erro na inicialização do Tcp/Ip.'
        );
        break;
      case 7:
        $retorno = array(
          'error' => true,
          'message' => 'Falta de memória.'
        );
        break;
      case 8:
        $retorno = array(
          'error' => true,
          'message' => 'Não encontrou a CliSiTef ou ela está com problemas.'
        );
        break;
      case 10:
        $retorno = array(
          'error' => true,
          'message' => 'Erro de acesso na pasta CliSiTef (Possível falta de permissão para escrita).'
        );
        break;
      default:
        $retorno = array(
          'error' => true,
          'message' => 'Falha ao configurar o Cliente SiTef.'
        );
        break;
    }
    return $retorno;
  }

  public function continuaTransacaoTef($valorTotal, $IDTIPORECE, $CDTIPORECE, $DATECANCEL, $DOCNUMBER)
  {
    try {
      $errorMessage = '';
      $comando = 0;
      $tipoCampo = 0;
      $tamMinimo = 0;
      $tamMaximo = 0;
      $bufferParam = '';
      $tamBuffer = 300;
      $continua = 0;
      do {
        $paramsContinua = array(
          'command'           => $comando,
          'fieldType'         => $tipoCampo,
          'minSize'           => $tamMinimo,
          'maxSize'           => $tamMaximo,
          'bufferParam'       => $bufferParam,
          'sizeBuffer'        => $tamBuffer,
          'continueCommand'   => $continua
        );
        $this->setClientIpCurl();
        $retornoContinua = $this->curl->request(self::EVENT_CONTINUA_TRANSACAO, $paramsContinua);
        $retornoContinua = json_decode($retornoContinua, true);
        $bufferMessage = mb_convert_encoding(json_decode($retornoContinua['result'], true)['bufferConverted'], 'UTF-8', 'UTF-8');
        $bufferMessage = trim($bufferMessage);
        $retornoContinua = json_decode($retornoContinua['result'], true);
        $retornoContinua['bufferConverted'] = $bufferMessage;
        Exception::logException(new \Exception($bufferMessage . 'Comando: ' . $retornoContinua['comando'] . ' / TipoCampo: ' . $retornoContinua['tipoCampo'] . ' / tamMinimo: ' . $retornoContinua["tamMinimo"] . ' / tamMaximo: ' . $retornoContinua["tamMaximo"]), Exception::LOG_TYPES['EXCEPTION_LOG']);
        if ($retornoContinua['retornoDll'] == 10000) {
          switch ($retornoContinua['comando']) {
            case 0:
              switch ($retornoContinua['tipoCampo']) {
                case 0:
                  $retornoContinua['bufferConverted'] = '';
                  break;
                case 15:
                  //Cartão de débito (todas as combinações)(Descontinuado, não usar)
                  $retornoContinua['bufferConverted'] = '';
                  break;
                case 43:
                  // echo '<p>Débito Magnético:'.$retornoContinua['bufferConverted'].' </p>';
                  $retornoContinua['bufferConverted'] = '';
                  //Débito Magnético
                  break;
                case 100:
                  // echo '<p>Contém a modalidade de pagamento no formato xxnn xx corresponde ao grupo da modalidade  e  nn  ao  sub-grupo.  Vide  tabela  no  final  deste  documento  descrevendo os possíveis valores de xx e nn.</p>';
                  break;
                case 101:
                  // echo '<p>Contém  o  texto  real  da  modalidade  de  pagamento  que  pode  ser  memorizado  pela aplicação caso exista essa necessidade. Descreve por extenso o par xxnnfornecido em 100</p>';
                  break;
                case 102:
                  // echo '<p>Contém  o  texto  descritivo  da  modalidade  de  pagamento  que  deve  ser  impresso  no cupon fiscal (p/ex: T.E.F., Cheque, etc...)</p>';
                  break;
                case 105:
                  // echo '<p>Contém a data e hora da transação no formato AAAAMMDDHHMMSS</p>';
                  break;
                case 110:
                  break;
                case 111:
                  break;
                  break;
                case 121:
                  //Buffer  contém  a  primeira  via do  comprovante de  pagamento  (via  do  cliente)  a  ser impressa  na  impressora  fiscal. Essa  via,  quando  possível,  é  reduzida  de  forma  a ocupar  poucas  linhas  na  impressora.  Pode  ser  um  comprovante  de  venda  ou administrativo
                  // echo '<p>Buffer  contém  a  primeira  via do  comprovante de  pagamento  (via  do  cliente)  a  ser impressa  na  impressora  fiscal. Essa  via,  quando  possível,  é  reduzida  de  forma  a ocupar  poucas  linhas  na  impressora.  Pode  ser  um  comprovante  de  venda  ou administrativo</p>';
                  self::salvaDadoTemporario('comprovanteTef', $retornoContinua['bufferConverted']);
                  $retornoContinua['bufferConverted'] = '';
                  break;
                case 122:
                  //Buffer  contém  a  segunda  via  do  comprovante  de  pagamento  (via  do  caixa)  a  ser impresso   na   impressora   fiscal.   Pode   ser   um   comprovante   de   venda   ou administrativo
                  // echo '<p>Buffer  contém  a  segunda  via  do  comprovante  de  pagamento  (via  do  caixa)  a  ser impresso   na   impressora   fiscal.   Pode   ser   um   comprovante   de   venda   ou administrativo</p>';
                  $retornoContinua['bufferConverted'] = '';
                  break;
                case 123:
                  /* Tipos:
                                    COMPROVANTE_COMPRAS = "00"COMPROVANTE_VOUCHER = "01"
                                    COMPROVANTE_CHEQUE = "02"COMPROVANTE_PAGAMENTO= "03"COMPROVANTE_GERENCIAL= "04"COMPROVANTE_CB= "05"COMPROVANTE_RECARGA_CELULAR= "06"COMPROVANTE_RECARGA_BONUS= "07"COMPROVANTE_RECARGA_PRESENTE= "08"COMPROVANTE_RECARGA_SP_TRANS= "09"COMPROVANTE_MEDICAMENTOS= "10"
                                    */
                  // echo '<p>Indica  que  os  comprovantes  que  serão  entregues  na  seqüência  são  de  determinado tipo (no comentario)</p>';
                  $retornoContinua['bufferConverted'] = '';
                  break;
                case 131: // Contém as 6 primeiras posições do cartão (bin)
                  // echo '<p>Contém;  um  índice  que  indica  qual  a  instituição  que  irá  processar  a  transação segundo a tabela presente no final do documento (5 posições):'.$retornoContinua['bufferConverted'].' </p>';
                  break;
                case 132:
                  // Tipo cartão
                  // echo '<p>Contém um índice que indica qual o tipo do cartão quando esse tipo for identificável, segundo uma tabela a ser fornecida (5 posições):'.$retornoContinua['bufferConverted'].' </p>';
                  self::salvaDadoTemporario('CDBANCARTCR', $retornoContinua['bufferConverted']);
                  break;
                case 133:
                  // Tipo cartão
                  // echo 'NSU Sitef: (6 posições): ' . $retornoContinua['bufferConverted'];
                  self::salvaDadoTemporario('CDNSUHOSTTEF', $retornoContinua['bufferConverted']);
                  break;
                case 134:
                  // Tipo cartão
                  // echo 'NSU do Host autorizador (15 posições no máximo): ' . $retornoContinua['bufferConverted'];
                  break;
                case 135:
                  // Tipo cartão
                  // echo 'Contém  o  Código  de  Autorização  para  as  transações  de  crédito  (15  posições  no máximo)'.$retornoContinua['bufferConverted'].' </p>';
                  break;
                case 136: // Contém as 6 primeiras posições do cartão (bin)
                  // echo '<p>Contém as 6 primeiras posições do cartão (bin):'.$retornoContinua['bufferConverted'].' </p>';
                  break;
                case 146:
                  break;
                case 147:
                  break;
                case 156:
                  // echo '<p>Nome da instituição:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 157:
                  // echo '<p>Código de Estabelecimento:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 158:
                  // echo '<p>Código da Rede Autorizadora –Serviço H:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 161:
                  // echo '<p>Número Identificador do Cupom do Pagamento:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 170:
                  // echo '<p>Venda Parcelada Estabelecimento Habilitada:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 171:
                  // echo '<p>Número Mínimo de Parcelas –Parcelada Estabelecimento:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 172:
                  // echo '<p>Número Máximo de Parcelas –Parcelada Estabelecimento:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 173:
                  // echo '<p>Valor Mínimo Por Parcela –Parcelada Estabelecimento:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 174:
                  // echo '<p>Venda Parcelada Administradora Habilitada:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 175:
                  // echo '<p>Número Mínimo de Parcelas –Parcelada Administradora:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 176:
                  // echo '<p>Número Máximo de Parcelas –Parcelada Administradora:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 724:
                  // echo '<p>Venda Crédito Parcelada com Plano Habilitada:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 725:
                  // echo '<p>Venda Crédito com Autorização a Vista Habilitada:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 726:
                  // echo '<p>Venda Crédito com Autorização Parcela com Plano Habilitada:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 727:
                  // echo '<p>VendaBoleto Habilitada:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 2010:
                  // echo '<p>Código de resposta do autorizador:' . $retornoContinua['bufferConverted'] . ' </p>';
                  break;
                case 2090:
                  //Tipo do cartão Lido
                  // echo '<p>Tipo do cartão Lido:'.$retornoContinua['bufferConverted'].' </p>';
                  break;
                case 2091:
                  //Tipo do cartão Lido
                  // echo '<p>Status da última leitura do cartão:'.$retornoContinua['bufferConverted'].' </p>';
                  break;
                case 1:
                case 2:
                case 504:
                  $retornoContinua['bufferConverted'] = '0';
                  break;
                case 800:
                case 801:
                case 951:
                case 952:
                case 953:
                case 1002:
                case 1003:
                case 1190:
                case 2053:
                case 2333:
                case 2362:
                case 2364:
                case 2421:
                case 4100:
                case 5049:
                case 5074:
                case 1321:
                  //Tipo do cartão Lido
                  // echo '<p>Valor retornoContinua[.tipoCampo] não documentado, buffer:'.$retornoContinua['bufferConverted'].' </p>' ;
                  break;
                default:
                  if ($retornoContinua['tipoCampo'] >= 10 && $retornoContinua['tipoCampo'] <= 99) {
                    // echo '<p>Tipo do cartão: ' . $retornoContinua['tipoCampo'] . '</p>';
                  } else {
                    return array(
                      'error' => true,
                      'message' => 'Error retornoContinua[tipoCampo] invalido :' . $retornoContinua['tipoCampo'],
                      'finishedTransaction' => true
                    );
                  }
                  break;
              }
              break;
            case 1:
              //Mensagem para o visor do operador
              $retornoContinua['bufferConverted'] = '';
              break;
            case 2:
              $retornoContinua['bufferConverted'] = '';
              break;
            case 3:
              //"Mensagem para os dois visores"
              // echo '<p>Mensagem para os dois visores :'.$retornoContinua['bufferConverted'].' </p>';
              $message = mb_convert_encoding($retornoContinua['bufferConverted'], 'UTF-8', 'UTF-8');
              $message = $errorMessage . '! ' . trim($message);
              $retornoContinua['bufferConverted'] = '';
              return array(
                'error' => false,
                'finishedTransaction' => false,
                'message' => $message
              );
              break;
            case 4:
              break;
            case 13:
              //LimpaMensagem;
              $retornoContinua['bufferConverted'] = '';
              break;
            case 14:
              // echo '<p>Deve limpar o texto utilizado como cabeçalho na apresentação do menu</p>';
              break;
            case 15:
              //Cabeçalho a ser apresentado pela aplicação?
              $retornoContinua['bufferConverted'] = '';
              break;
            case 20:
              //Deve  obter  uma  resposta  do  tipo  SIM/NÃO.  No  retorno  o  primeiro  caráter presente em Buffer deve conter 0 se confirma e 1 se cancela
              //na primeira vez q caiu aqui foi por erro de leitura.
              //tem que mandar a pegunta na tela e ver se o cara quer continuar
              //$confirma = $this->mensagemConfirma();// '0' se ok, '1' se nao ok
              $confirma = '0';
              $retornoContinua['bufferConverted'] = $confirma;
              break;
            case 21:
              // echo '<p>Deve apresentar um menude opções e permitir que o usuário selecione uma delas. Na chamada o parâmetro Buffer contém as opções no formato 1:texto;2:texto;...i:Texto;...  A  rotina  da  aplicação  deve  apresentar  as  opções  da forma  que  ela  desejar  (não  sendo  necessário  incluir  os  índices  1,2,  ...)  e  após  a seleção  feita  pelo  usuário,  retornar  em  Buffer  o  índice  i  escolhido  pelo  operador (em ASCII)</p>';
              // Simula a vista
              $confirma = '1';
              $retornoContinua['bufferConverted'] = $confirma;
              break;
            case 22:
              /* Deve aguardar uma tecla do operador. É utilizada quando se deseja que o operador seja avisado de alguma mensagem apresentada na tela */
              $message = mb_convert_encoding($retornoContinua['bufferConverted'], 'UTF-8', 'UTF-8');
              $message = trim($message);
              $retornoContinua['bufferConverted'] = '';
              $errorMessage = $message;
              break;
            case 23:
              // Este comando indica que a rotina está perguntando para a aplicação se ele deseja interromper  o   processo de coleta de dados ou não. Esse código ocorre quando a CliSiTefestá  acessando  algum  periférico  e  permite  que  a  automação  interrompa esse acesso (por exemplo: aguardando a passagem de um cartão pela leitora ou a digitação de senha pelo cliente)
              //$retornoContinua['retornoDll'] := -5;
              //$retornoContinua['comando'] = -1;
              $retornoContinua['bufferConverted'] = '';
              break;
            case 30:
              // Deve  ser  lido  um  campo  cujo  tamanho  está  entre TamMinimo e TamMaximo. O campo lido deve ser devolvido em Buffer
              // return array(
              //     'error' => true,
              //     'message' =>  'Cartão inválido para este tipo de operação.',
              //     'finishedTransaction' => true
              // );
              switch ($retornoContinua['tipoCampo']) {
                case 515:
                  $retornoContinua['bufferConverted'] = $DATECANCEL;
                  break;

                case 516:
                  $retornoContinua['bufferConverted'] = (int)$DOCNUMBER;
                  break;
              }
              break;
            case 34:
              switch ($retornoContinua['tipoCampo']) {
                case 146:
                  $retornoContinua['bufferConverted'] = $valorTotal;
                  break;
              }
              //Deve ser lido um campo monetário ou seja, aceita o delimitador de centavos e devolvido no parâmetro Buffer
              break;
            default:
              return array(
                'error' => true,
                'message' =>  'Error retornoContinua[comando] invalido :' . $retornoContinua['comando'],
                'finishedTransaction' => true
              );
              break;
          }

          $comando        = $retornoContinua["comando"];
          $tipoCampo      = $retornoContinua['tipoCampo'];
          $tamMinimo      = $retornoContinua["tamMinimo"];
          $tamMaximo      = $retornoContinua["tamMaximo"];
          $tamBuffer      = 300;
          $bufferParam    = $retornoContinua['bufferConverted'];
          $continua       = $retornoContinua["continua"];
        } elseif ($retornoContinua['retornoDll'] > 0) {
          // Encerra transação
          $paramsContinua = array(
            'command'           => 0,
            'fieldType'         => 0,
            'minSize'           => 0,
            'maxSize'           => 0,
            'bufferParam'       => '',
            'sizeBuffer'        => 0,
            'continueCommand'   => -1
          );
          $this->setClientIpCurl();
          $retornoContinua = $this->curl->request(self::EVENT_CONTINUA_TRANSACAO, $paramsContinua);
          return array(
            'error' => true,
            'message' => $errorMessage
          );
        }
      } while ($retornoContinua['retornoDll'] == 10000);

      if ($retornoContinua['retornoDll'] == 0) {
        $dadosTransacao = self::carregaDadosTransacao();
        $CDTIPORECE = self::trataTipoRece($CDTIPORECE, $dadosTransacao['CDBANCARTCR'], $IDTIPORECE);

        // Salva log final da transação
        if (self::SALVA_LOGS) {
          self::salvaEtapaTransacao('FINALIZANDO TRANSACAO', '');
          self::salvaEtapaTransacao('CDNSUHOSTTEF', $dadosTransacao['CDNSUHOSTTEF']);
          self::salvaEtapaTransacao('comprovanteTef', $dadosTransacao['comprovanteTef']);
          self::salvaEtapaTransacao('CDBANCARTCR', $dadosTransacao['CDBANCARTCR']);
        }

        return array(
          'error' => false,
          'CDNSUHOSTTEF' => $dadosTransacao['CDNSUHOSTTEF'],
          'COMPROVANTETEF' => $dadosTransacao['comprovanteTef'],
          'CDTIPORECE' => $CDTIPORECE,
          'finishedTransaction' => true
        );
      } else {
        switch ($retornoContinua['retornoDll']) {
          case -2:
            return array(
              'error' => true,
              'canceled' => true,
              'message' => 'Cancelado pelo operador.',
              'finishedTransaction' => true
            );
          case -15:
            return array(
              'error' => true,
              'message' => 'Sem comunicação com o SiTef.',
              'finishedTransaction' => true
            );
            break;
          case -43:
            return array(
              'error' => true,
              'message' => 'Falha no leitor de cartões.',
              'finishedTransaction' => true
            );
            break;
          default:
            return array(
              'error' => true,
              'message' => 'Erro desconhecido. Erro: ' . $retornoContinua['retornoDll'] . '.',
              'finishedTransaction' => true
            );
            break;
        }
      }
    } catch (\Exception $e) {
      Exception::logException($e, Exception::LOG_TYPES['EXCEPTION_LOG']);
      return array(
        'error' => true,
        'message' => 'Mensagem: ' . $e->getMessage(),
        'finishedTransaction' => true
      );
    }
  }

  public function finalizaTransacao($confirma)
  {
    try {
      $paramsContinua = array(
        'confirm' => $confirma,
        'text' => '',
        'date' => date('Ymd'),
        'time' => date('His')
      );
      $this->setClientIpCurl();
      $retornoContinua = $this->curl->request(self::EVENT_FINALIZA_TRANSACAO, $paramsContinua);
      return array(
        'error' => false,
        'message' => 'Transação realizada com sucesso.',
        'finishedTransaction' => true
      );
    } catch (\Exception $e) {
      return array(
        'error' => false,
        'message' => 'Transação realizada com sucesso.',
        'finishedTransaction' => true
      );
    }
  }

  private function salvaDadoTemporario($field, $value)
  {
    $this->util->createFolder($this->systemPath . 'TEF/TEMP');
    $fileName = 'temp_' . $field . '.txt';
    $impFile = fopen($this->systemPath . 'TEF/TEMP/' . $fileName, 'w');
    fwrite($impFile, $value);
    fclose($impFile);
  }

  private function carregaDadosTransacao()
  {
    $tempDir = $this->systemPath . 'TEF/TEMP/temp_';

    $CDBANCARTCRFile = fopen($tempDir . 'CDBANCARTCR.txt', 'r');
    $CDBANCARTCR = fread($CDBANCARTCRFile, filesize($tempDir . 'CDBANCARTCR.txt'));

    $CDNSUHOSTTEFFile = fopen($tempDir . 'CDNSUHOSTTEF.txt', 'r');
    $CDNSUHOSTTEF = fread($CDNSUHOSTTEFFile, filesize($tempDir . 'CDNSUHOSTTEF.txt'));

    $comprovanteTefFile = fopen($tempDir . 'comprovanteTef.txt', 'r');
    $comprovanteTef = fread($comprovanteTefFile, filesize($tempDir . 'comprovanteTef.txt'));

    return array(
      'CDBANCARTCR' => $CDBANCARTCR,
      'CDNSUHOSTTEF' => $CDNSUHOSTTEF,
      'comprovanteTef' => $comprovanteTef,
    );
  }

  private function trataTipoRece($CDTIPORECE, $CDBANCARTCR, $IDTIPORECE)
  {
    $params = array(
      'CDBANCARTCR' => $CDBANCARTCR,
      'IDTIPORECE' => $IDTIPORECE
    );
    $tipoRece = $this->entityManager->getConnection()->fetchAssoc("BUSCA_TIPO_RECE", $params);
    if (!empty($tipoRece)) {
      $CDTIPORECE_utilizado = $tipoRece['CDTIPORECE'];
    } else {
      $CDTIPORECE_utilizado = $CDTIPORECE;
    }
    return $CDTIPORECE_utilizado;
  }

  private function salvaEtapaTransacao($field, $value)
  {
    $this->util->createFolder($this->systemPath . 'TEF/LOGS');
    $fileName = 'saveStep_transaction' . date('d-m-Y_H') . '.txt';

    $impFile = fopen($this->systemPath . 'TEF/LOGS/' . $fileName, 'a');

    $currentDateTime = date('d-m-Y_H-i-s');
    $transactionData = $currentDateTime . ' - ' . $field . ': ' . $value . "\r\n";

    fwrite($impFile, $transactionData);

    fclose($impFile);
  }

  public function validaParametrosTef($CDFILIAL, $CDCAIXA)
  {
    $retorno = array();

    $params = array(
      'CDFILIAL' => $CDFILIAL,
      'CDCAIXA' => $CDCAIXA
    );

    $dadosTef = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_TEF", $params);
    if (empty($dadosTef['CDLOJATEF'])) {
      $retorno = array(
        'error' => true,
        'message' => 'Loja TEF não parametrizado para o caixa.'
      );
    } elseif (empty($dadosTef['CDTERTEF'])) {
      $retorno = array(
        'error' => true,
        'message' => 'Terminal TEF não parametrizado para o caixa.'
      );
    } elseif( isset($dadosTef['IDTPTEF']) && $dadosTef['IDTPTEF'] === '2') {
      $retorno = array(
        'error' => false
      );
    } elseif (empty($dadosTef['DSENDIPSITEF'])) {
      $retorno = array(
        'error' => true,
        'message' => 'Endereço IP do servidor TEF não parametrizado para o caixa.'
      );
    } else {
      $retornoConfigura = $this->configuraTransacaoTef($CDFILIAL, $CDCAIXA);
      if (!!$retornoConfigura['error']) {
        $retorno = $retornoConfigura;
      } else {
        $retorno = array(
          'error' => false,
          'tefConfigurationParams' => $dadosTef
        );
      }
    }
    return $retorno;
  }

  private function setClientIpCurl()
  {
    if (isset($_SERVER['HTTP_CLIENT_IP']))
      $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
    else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
      $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
    else if (isset($_SERVER['HTTP_X_FORWARDED']))
      $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
    else if (isset($_SERVER['HTTP_FORWARDED_FOR']))
      $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
    else if (isset($_SERVER['HTTP_FORWARDED']))
      $ipaddress = $_SERVER['HTTP_FORWARDED'];
    else if (isset($_SERVER['REMOTE_ADDR']))
      $ipaddress = $_SERVER['REMOTE_ADDR'];

    $this->curl->setBaseUrl('http://' . $ipaddress . ':3000');
  }
}
