<?php
/**
 * GepTipoFluxoLogic.
 *
 * @package Logic
 * @author  daniellepereira <username@example.com>
 */
class GepTipoFluxoLogic{


    /**
     * Gera novo c?o de Tipo de Fluxo.
     *
     * @param integer $nrorg N?o da Organiza?.
     *
     * @return $codigo integer
     */
    public static function retornaProxNumeroTipoFluxo($nrorg){
        $dados = GepTipofluxoEntity::retornaProxNumeroTipoFluxo($nrorg);
        return $dados;
    }


    /**
     * Retorna a consulta de parceiros de negocio de um ou mais tipos de parceria que se encontram em um tipo de fluxo.
     *
     * @return string              Resultado da query
     */
    public static function buscaParceirosPorTipoFluxo($nrorg, $tipofluxo){
        if (empty($tipofluxo) === true) {
            $tipofluxo = '';
        } else {
            $tipofluxo = "JOIN GEP_TIPOPARCFLUXO
                          ON GEP_TIPOPARCFLUXO.NRTIPOFLUXO     = $tipofluxo
                          AND GEP_TIPOPARCFLUXO.NRORG          = PARCNEGOCIO.NRORG
                          AND GEP_TIPOPARCFLUXO.CDTIPOPARCEIRO = PC.CDTIPOPARCEIRO";
        }

        $sql = GepTipofluxoEntity::buscaParceirosPorTipoFluxo($nrorg, $tipofluxo);
        return $sql;
    }


    /**       ok
    * Retorna os parametros do tipo de fluxo informado.
    *
    * @param integer $nrorg       Numero da organizacao.
    * @param integer $nrtipofluxo Numero do tipo de fluxo.
    *
    * @return $dados
    */
    public static function retornaTipoFluxo($nrorg,$nrtipofluxo){
        try {
            $dados = GepTipofluxoEntity::retornaParametrosTipoFluxo(
                $nrorg,
                $nrtipofluxo
            );
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()){
            case GepTipofluxoEntity::SEM_REGISTRO:
                return GepTipofluxoEntity::SEM_REGISTRO;
            default:
                tk_warning('Erro desconhecido: '.$e->getMessage());
                return false;
            }
        }
    }


    /**    ok
     * Retorna o texto da descricao da solicicao formatado.
     *
     * @param integer $nrorg    Nao da Organizacao.
     * @param integer $tp_fluxo Tipo de fluxo.
     *
     * @return string|boolean
     */
    public static function retornaTextoFormat($nrorg, $tp_fluxo){
        $txt_help = self::retornaTipoFluxo($nrorg, $tp_fluxo);

        if ($txt_help !== false AND $txt_help !== GepTipofluxoEntity::SEM_REGISTRO) {
            $txt_help = $txt_help->getDsdescricaosoli();
            if ($txt_help !== null) {
                $texto = nl2br($txt_help);
            } else {
                $texto = "Descreva o mais detalhado possível o erro para obter uma solução rápida.";
            }
        } else {
            $texto = "Descreva o mais detalhado possível o erro para obter uma solução rápida.";
        }

        $txt_format = '<br><div style= "background:none repeat scroll 0% 0%; white-space:normal;
                				margin:0px; width:860px; height:auto;" >'.$texto."</div><br>";
        return $txt_format;

    }


    /**
     * Retorna o nome da window que e filtrada pelo tipo de fluxo informado.
     *
     * @param integer $tipofluxo Tipo de Fluxo.
     *
     * @return string
     */
    public function selecionaTelaSoliPorTipoFluxo($tipofluxo){
        $window_name = 'sf_gep50400_'.$tipofluxo;
        return $window_name;
    }


     static function verificaAcesso($nrorg, $tipofluxo, $parceiro){
        $resultado = GepTipofluxoEntity::verificaAcesso($nrorg, $tipofluxo, $parceiro);
        return $resultado;
    }

}